unit uPlot_ResultWriterHelperForms;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, StdCtrls, Controls, Grids, Spin,
  CsvDocument;

type

  TCSVVariant = (csvvSingleSet, csvvXMultiY);

  { THelperFormResultWriter }

  THelperFormResultRW = class(TForm)
    procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
  private
    FBtnOK: TButton;
    FDoneCallbackProc: TNotifyEvent;
  protected
    procedure OnBtnOK(Sender: TObject);virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    property DoneCallbackProc: TNotifyEvent read FDoneCallbackProc write FDoneCallbackProc;
  end;


  { THelperFormResultReaderCSV }

  THelperFormResultReaderCSV = class(THelperFormResultRW)
  private
    Fsg_preview: TStringGrid;
    Fcb_variant: TComboBox;
    Fcb_delimiter: TComboBox;
    Flbl_headerlines: TLabel;
    Fed_headerlines: TSpinEdit;
    FCSVDelimiter: Char;
    FCSVNumHeaderLines: Integer;
    FCSVvariant: TCSVVariant;
    FCSVParser: TCSVParser;
    FLastFileName: TFilename;
    procedure DoPreView(AFileName: TFilename);
  protected
    procedure Apply(Sender: TObject);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure PreView(AFileName: TFilename);

    property CSVDelimiter: Char read FCSVDelimiter;
    property CSVNumHeaderLines: Integer read FCSVNumHeaderLines;
    property CSVvariant: TCSVVariant read FCSVvariant;
  end;

const
  cCSVVariantNames: array[csvvSingleSet..csvvXMultiY] of String = ('Single Dataset n dimensions','One X multi Y');

implementation

{ THelperFormResultReaderCSV }

procedure THelperFormResultReaderCSV.DoPreView(AFileName: TFilename);
var
  vFileStream: TFileStream;
  vActualRow: Integer;
begin
  vActualRow := 0;
  vFileStream := TFileStream.Create(AFilename, fmOpenRead+fmShareDenyWrite);
  try
    FCSVParser.Delimiter := FCSVDelimiter;
    FCSVParser.SetSource(vFileStream);

    while FCSVParser.ParseNextCell do begin
      // Stop if we've filled all existing rows. Todo: check for fixed grids etc, but not relevant for our case
      if FCSVParser.CurrentRow+1 > Fsg_preview.RowCount then break;
      // widen grid
      if Fsg_preview.ColCount<FCSVParser.CurrentCol+1 then Fsg_preview.ColCount:=FCSVParser.CurrentCol+1;

      Fsg_preview.Cells[FCSVParser.CurrentCol,FCSVParser.CurrentRow]:=FCSVParser.CurrentCellText;
      inc(vActualRow);
    end;
    // shrink grid
    if Fsg_preview.ColCount>FCSVParser.MaxColCount then
     Fsg_preview.ColCount:=FCSVParser.MaxColCount;
    Fsg_preview.AutoSizeColumns ;
  finally
    vFileStream.Free;
  end;

  //Fsg_preview.LoadFromCSVFile(AFileName, CSVDelimiter, true);
  //FCSVParser.;
end;

procedure THelperFormResultReaderCSV.Apply(Sender: TObject);
begin
  // TODO:
  Fsg_preview.FixedRows := Fed_headerlines.Value;
  FCSVNumHeaderLines := Fed_headerlines.Value;
  FCSVDelimiter := String( Fcb_delimiter.Items[Fcb_delimiter.ItemIndex])[1];
  FCSVvariant := TCSVVariant(Fcb_variant.ItemIndex);

  // refresh preview if delimiter changed
  if Sender = Fcb_delimiter then begin
    DoPreView(FLastFileName);
  end;
end;

constructor THelperFormResultReaderCSV.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FCSVParser := TCSVParser.Create;
  FCSVNumHeaderLines := 0;
  FCSVDelimiter := ',';
  FCSVvariant := csvvSingleSet;

  // components
  Fcb_variant := TComboBox.Create(Self);
  with Fcb_variant do begin
    Parent := Self;
    //Caption := 'OK';
    Width := 182;
    Items.Add(cCSVVariantNames[csvvSingleSet]);
    Items.Add(cCSVVariantNames[csvvXMultiY]);
    ItemIndex := 0;
    BorderSpacing.Top := 6;
    BorderSpacing.Left := 6;
    AnchorSide[akLeft].Control := Self;
    AnchorSide[akLeft].Side := asrLeft;
    AnchorSide[akTop].Control := Self;
    AnchorSide[akTop].Side := asrTop;
    Anchors := [akTop, akLeft];
    OnChange := @Apply;
  end;

  Fcb_delimiter := TComboBox.Create(Self);
  with Fcb_delimiter do begin
    Parent := Self;
    //Caption := 'OK';
    Width := 36;
    Items.Add(',');
    Items.Add(';');
    Items.Add(':');
    ItemIndex := 0;
    BorderSpacing.Top := 0;
    BorderSpacing.Left := 6;
    AnchorSide[akLeft].Control := Fcb_variant;
    AnchorSide[akLeft].Side := asrRight;
    AnchorSide[akTop].Control := Fcb_variant;
    AnchorSide[akTop].Side := asrCenter;
    Anchors := [akTop, akLeft];
    OnChange := @Apply;
  end;

  Fed_headerlines := TSpinEdit.Create(Self);
  with Fed_headerlines do begin
    Parent := Self;
    //Caption := 'OK';
    MinValue := 0;
    MaxValue := 99;
    Value := 0;
    BorderSpacing.Top := 0;
    BorderSpacing.Left := 6;
    AnchorSide[akLeft].Control := Fcb_delimiter;
    AnchorSide[akLeft].Side := asrRight;
    AnchorSide[akTop].Control := Fcb_variant;
    AnchorSide[akTop].Side := asrCenter;
    Anchors := [akTop, akLeft];
    OnChange := @Apply;
  end;

  Flbl_headerlines := TLabel.Create(Self);
  with Flbl_headerlines do begin
    Parent := Self;
    Caption := 'discarded headerlines';
    BorderSpacing.Top := 0;
    BorderSpacing.Left := 6;
    AnchorSide[akLeft].Control := Fed_headerlines;
    AnchorSide[akLeft].Side := asrRight;
    AnchorSide[akTop].Control := Fcb_variant;
    AnchorSide[akTop].Side := asrCenter;
    Anchors := [akTop, akLeft];
  end;

  Fsg_preview := TStringGrid.Create(Self);
  with Fsg_preview do begin
    Parent := Self;
    FixedCols := 0;
    FixedRows := 0;
    RowCount := 6;
    //Caption := 'OK';
    BorderSpacing.Top := 6;
    BorderSpacing.Left := 6;
    AnchorSide[akLeft].Control := Self;
    AnchorSide[akLeft].Side := asrLeft;
    AnchorSide[akTop].Control := Fcb_variant;
    AnchorSide[akTop].Side := asrBottom;
    AnchorSide[akRight].Control := Self;
    AnchorSide[akRight].Side := asrRight;
    AnchorSide[akBottom].Control := FBtnOK;
    AnchorSide[akBottom].Side := asrTop;
    Anchors := [akTop, akLeft, akRight, akBottom];
    //OnChange := @Apply;
  end;
end;

destructor THelperFormResultReaderCSV.Destroy;
begin
  FCSVParser.Free;
  inherited Destroy;
end;

procedure THelperFormResultReaderCSV.PreView(AFileName: TFilename);
begin
  FLastFileName := AFileName;
  DoPreView(AFileName);
end;

{ THelperFormResultWriter }

procedure THelperFormResultRW.FormCloseQuery(Sender: TObject;
  var CanClose: boolean);
begin
  if assigned(FDoneCallbackProc) then FDoneCallbackProc(Self);
  CanClose := true;
end;

procedure THelperFormResultRW.OnBtnOK(Sender: TObject);
begin
  Self.Close;
end;

constructor THelperFormResultRW.Create(AOwner: TComponent);
const
  cBUTTON_GEN_SPACE = 20;
begin
  inherited CreateNew(AOwner);
  FDoneCallbackProc := nil;

  // generic buttons
  FBtnOK := TButton.Create(Self);
  FBtnOK.Parent := Self;
  FBtnOK.Caption := 'OK';
  FBtnOK.OnClick := @OnBtnOK;
  FBtnOK.BorderSpacing.Around := cBUTTON_GEN_SPACE;
  FBtnOK.AnchorSide[akLeft].Control := Self;
  FBtnOK.AnchorSide[akLeft].Side := asrLeft;
  FBtnOK.AnchorSide[akBottom].Control := Self;
  FBtnOK.AnchorSide[akBottom].Side := asrBottom;
  FBtnOK.Anchors := [akBottom, akLeft];

  Self.Width := 320;
  Self.Height := 240;
  Self.Caption := 'Import settings';
  Self.OnCloseQuery := @FormCloseQuery;
  //Self.ShowModal;
end;

destructor THelperFormResultRW.Destroy;
begin
  inherited Destroy;
end;

end.

